//******************************************************************************
// Projeto - SERVO CONTROL - controle de at oito servos atravs da RS-232
// Desenvolvido por Mrcio Jos Soares
//
// Arquivo - servo_control.c
//
// Aguarda dado do canal serial e compara com protocolo
// Comunicao: 9600, n, 8, 1
//
// Formato do protocolo:
// <STX><nr do servo - 1 a 8><posio 0 a 255><ETX>
//
// LED1 pisca a 1 Hz (1 seg): indicao de ligado (timer 0)
//
// Servos so operados a cada 20 ms (timer 1)
//
// Microcontrolador utilizado: PIC16F628-04/P (no testado com verso "A")
// Perifricos - servos Hobbico CS61, Futaba S148 e/ou Futaba S3003
// Compilador: CCS C Compiler - verso PCM 3.15
//
// Pinos utilizados para controle dos servos:
// RA1 - servo 1
// RA0 - servo 2
// RA7 - servo 3
// RA6 - servo 4
// RB7 - servo 5
// RB6 - servo 6
// RB5 - servo 7
// RB4 - servo 8
//
// Pinos utilizados para comunicao com PC:
// Obs:  utilizado MAX-232 para drive TTL/RS-232
// RB1 - RX - recepo dos dados
// RB2 - TX - sada dos dados (eco)
//
// Pino utilizado para controle do LED:
// RA2 - LED
//
//******************************************************************************

//******************************************************************************
// Arquivos includos neste mdulo
//******************************************************************************
#include "servo_control.h"

//******************************************************************************
// Funo para posicionar servos
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
void atualiza_servos(void){

	ubyte i;

	restart_WDT();               //restaura WDT

	output_high(srv1);				//insere pulso no servo 1
	i = t_srv1;
	while(i){							//prepara para temporizar - descontados lao,
		delay_us(5);					//chamada call e decremento de i.
		i--;
	}
	output_low(srv1);
	restart_WDT();               //restaura WDT

	output_high(srv2);				//insere pulso no servo 2
	i = t_srv2;
	while(i){							//prepara para temporizar - descontados lao,
		delay_us(5);					//chamada call e decremento de i.
		i--;
	}
	output_low(srv2);
	restart_WDT();               //restaura WDT

	output_high(srv3);				//insere pulso no servo 3
	i = t_srv3;
	while(i){							//prepara para temporizar - descontados lao,
		delay_us(5);					//chamada call e decremento de i.
		i--;
	}
	output_low(srv3);
	restart_WDT();               //restaura WDT

	output_high(srv4);				//insere pulso no servo 4
	i = t_srv4;
	while(i){							//prepara para temporizar - descontados lao,
		delay_us(5);					//chamada call e decremento de i.
		i--;
	}
	output_low(srv4);
	restart_WDT();               //restaura WDT

	output_high(srv5);				//insere pulso no servo 5
	i = t_srv5;
	while(i){							//prepara para temporizar - descontados lao,
		delay_us(5);					//chamada call e decremento de i.
		i--;
	}
	output_low(srv5);
	restart_WDT();               //restaura WDT

	output_high(srv6);				//insere pulso no servo 6
	i = t_srv6;
	while(i){							//prepara para temporizar - descontados lao,
		delay_us(5);					//chamada call e decremento de i.
		i--;
	}
	output_low(srv6);
	restart_WDT();               //restaura WDT

	output_high(srv7);				//insere pulso no servo 7
	i = t_srv7;
	while(i){							//prepara para temporizar - descontados lao,
		delay_us(5);					//chamada call e decremento de i.
		i--;
	}
	output_low(srv7);
	restart_WDT();               //restaura WDT

	output_high(srv8);				//insere pulso no servo 8
	i = t_srv8;
	while(i){							//prepara para temporizar - descontados lao,
		delay_us(5);					//chamada call e decremento de i.
		i--;
	}
	output_low(srv8);
	restart_WDT();               //restaura WDT

}


//******************************************************************************
// Funo para tratar a interrupo para dado recebido RS-232
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
#int_RDA
RDA_isr() {

   static ubyte carac;

   restart_WDT();                //restaura WDT
   carac = rxreg;                //colhe dado do canal serial
	putc(carac);

	switch(maq_est){					//analisa maquina de estado
		restart_WDT();             //restaura WDT
		case 0:							// inicio do protocolo?
			if (carac == STX){		//foi STX?
				printf("ok\n\r");
				maq_est = 1;			//sim, prximo caracter ser nr do servo
			}
			else{
				printf("err\n\r");	//envia msg de erro
				maq_est = 0;									//volta ao inicio
			}
			break;
		case 1:										// a vez do nmero do servo
			if((carac > 0) && (carac < 8)){	//esta dentro da faixa?
				nr_servo_atual = carac;			//guarda dado
				maq_est = 2;						//prximo caracter ser a posio
				printf("ok\n\r");
			}
			else{													//valor fora da faixa
				printf("err\n\r");	//envia msg de erro
				maq_est = 0;									//volta ao inicio
			}
			break;
		case 2:												// a vez da posio
			if((carac >= 1) && (carac <= 255)){		//esta dentro da faixa?
				pos_servo_atual = carac;				//guarda dado
				maq_est = 3;								//prximo caracter ser a posio
				printf("ok\n\r");
			}
			else{
				printf("err\n\r");	//envia msg de erro
				maq_est = 0;									//volta ao inicio
			}
			break;
		case 3:
			if (carac == ETX){								//recebeu fim do protocolo?
				printf("ok\n\r");
				switch (nr_servo_atual){					//ento atualiza vars
					case 1:										// servo 1?
						t_srv1 = pos_servo_atual;
						break;
					case 2:										// servo 2?
						t_srv2 = pos_servo_atual;
						break;
					case 3:										// servo 3?
						t_srv3 = pos_servo_atual;
						break;
					case 4:										// servo 4?
						t_srv4 = pos_servo_atual;
						break;
					case 5:										// servo 5?
						t_srv5 = pos_servo_atual;
						break;
					case 6:										// servo 6?
						t_srv6 = pos_servo_atual;
						break;
					case 7:										// servo 7?
						t_srv7 = pos_servo_atual;
						break;
					case 8:										// servo 8?
						t_srv8 = pos_servo_atual;
						break;
					default:
						break;
				}
				maq_est = 0;			//volta para inicio
			}
			else{
				printf("err\n\r");	//envia msg de erro
				maq_est = 0;									//volta ao inicio
			}
			break;
		default:
			printf("err\n\r");	//envia msg de erro
			maq_est = 0;									//volta ao inicio
	}

   restart_WDT();                //restaura WDT
}

//******************************************************************************
// Funo para tratar a interrupo do timer0 - controle do LED
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
#int_timer0
void trata_int_t0(){

   static ubyte contador;
   static boolean saida_led;

   restart_WDT();							//zera WDT

   //reinicia o timer0 em 131, mais a contagem que j passou
   set_timer0(131+get_timer0());
   contador++;
   //verifica se j ocorreram 125 interrupes
   //8 ms x 125 = 1 s
   if (contador == 125){
      contador = 0;                 //zera contador
      saida_led = !saida_led;       //troca estado do LED (var)
		output_bit(LED1,saida_led);   //insere no pino de I/O novo estado
   }

   restart_WDT();							//zera WDT
}

//******************************************************************************
// Funo para tratar a interrupo do timer1 - controle dos servos
// recebe 	- nada
// retorna 	- nada
//******************************************************************************
#int_timer1
void trata_int_t1(){

	restart_WDT();							//zera WDT

	//reinicia timer1 com 3096, mais contagem que passou
	set_timer1(63036+get_timer1());
   atualiza_servos();					//atualiza servos
	restart_WDT();							//zera WDT
}

//******************************************************************************
// Funo principal
//******************************************************************************
void main(void) {

	ubyte i;

   set_tris_A(0b00010000);		//configura pinos do port A como sada,
                              //exceto RA0
   set_tris_B(0b00001010);		//configura pinos do port B como sada,
                              //exceto RB1 e RB3

   setup_comparator(nc_nc_nc_nc); //desabilita comparadores internos
	setup_wdt(WDT_2304MS);			 //habilita watch dog timer a cada 2 segundos
	restart_WDT();						//zera WDT

   //timer 0 habilitado para controle do LED
	//LED deve piscar com frequncia igual a 1 Hz
   //[(4MHz/4)/64]/125 = 125 Hz -> valor da frequencia resultante
	//(T = 1/F -> T = 1/125 = 8 ms) ===> como 1 s / 8 ms = 125
	//a funo de tratamento da int timer0 deve conter contador (0 a 125)
	//para chegar ao tempo desejado. Para obter o valor de inicializao
	//do timer deve-se fazer (256-125)=131
   setup_timer_0(RTCC_INTERNAL | RTCC_DIV_64);
   set_timer0(131);

	//timer 1 habilitado para controle dos servos
   //servos devem receber pulsos a cada 20 ms
   //Ento:
	//[(4MHz/4)/8]/2500 = 50 Hz -> (T = 1/F -> T = 1/50 = 20 ms)
   //Valor inicial do timer1 (65536-2500)=63036
   setup_timer_1(T1_INTERNAL|T1_DIV_BY_8);
	set_timer1(63036);
	restart_WDT();								//zera WDT

	//timer2 desabilitado
	setup_timer_2(T2_DISABLED,0,1);
	restart_WDT();								//zera WDT

	t_srv1 = 125;						//inicia variveis
	t_srv2 = 125;
	t_srv3 = 125;
	t_srv4 = 125;
	t_srv5 = 125;
	t_srv6 = 125;
	t_srv7 = 125;
	t_srv8 = 125;

   //habilita interrupes
   enable_interrupts(INT_RDA);         //habilita interrupo RS-232 (rec)
   enable_interrupts(INT_TIMER0);      //habilita interrupo do timer0
	enable_interrupts(INT_TIMER1);      //habilita interrupo do timer1
   enable_interrupts(GLOBAL);          //habilita interrupo global
   restart_WDT();							   //zera WDT

	for (i=0;i<100;i++){						//aguarda 1 segundo
		delay_ms(10);
		restart_WDT();
	}

   printf("P\n\r");                    //envia sinal de prontido via RS-232
   restart_WDT();	                     //zera WDT

	if (!input (JP1))               		//colhe pino RB3, velocidade default 2400
      set_uart_speed(9600);        		//com jumper muda para 9600
		
//******************************************************************************
// Inicio do lao principal
// Aguarda dado via RS-232 atravs de interrupo
// O piscar do LED  controlado via interrupo do timer 0
// O controle dos servos  feito via interrupo do timer 1
//******************************************************************************

	maq_est = 0;						//zera mquina de estado para rec. protocol

	while(true){                  //faz eternamente
      restart_WDT();

   }
}
