//******************************************************************************
// Projeto - SERVO_CONTROL - controle de at oito servos via RS-232
// Desenvolvido por Mrcio Jos Soares
//
// Arquivo - servo_control.h - configuraes, definies e variveis globais
//******************************************************************************

//******************************************************************************
// Arquivos includos neste mdulo
//******************************************************************************
#include <16F628.h>

//******************************************************************************
// Configuraes para o microcontrolador PIC16F628
//******************************************************************************
#use delay(clock=4000000,RESTART_WDT)

//******************************************************************************
// Configurao dos fuses do microcontrolador PIC16F628
//******************************************************************************
#fuses INTRC_IO,PUT,WDT,NOBROWNOUT,NOMCLR,NOLVP,NOCPD
//INTRC	      Clock interno
//PUT          Ativa tempo de inicializao
//WDT		      Ativa Watch Dog Timer
//NOBROWNOUT	No usa brownout reset
//NOMCLR		   Pino Master Clear usado como I/O
//NOLVP		   No usa baixa voltagem para programao, pino B3 usado como I/O
//NOCPD		   No protege EEPROM

//******************************************************************************
// Configurao da comunicao serial do PIC16F628
//******************************************************************************
#use rs232(baud=2400,parity=N,xmit=PIN_B2,rcv=PIN_B1,bits=8)

//******************************************************************************
// Prioridade desejada para interrupes no PIC
//******************************************************************************
#priority timer1, RDA, timer0

//******************************************************************************
// Definies do usurio
//******************************************************************************
#define STX	0x02						//cdigo ASCII para STX - necessrio p/ protocol.
#define ETX	0x03						//cdigo ASCII para ETX

//******************************************************************************
// Definies para o mdulo
//******************************************************************************
#locate rxreg     = 0x1a         //endereo do registrador p/ byte recebido

#define srv1      PIN_A1         //servos e sua ligao com os pinos de I/O
#define srv2      PIN_A0
#define srv3      PIN_A7
#define srv4      PIN_A6
#define srv5      PIN_B7
#define srv6      PIN_B6
#define srv7      PIN_B5
#define srv8      PIN_B4

#define LED1      PIN_A2         //LED
#define JP1    	PIN_B3         //jumper para selecionar velocidade de comunicao

#define ubyte	unsigned char

//******************************************************************************
// Variveis Globais do mdulo
//******************************************************************************
ubyte t_srv1;				//tempo do pulso para servo 1
ubyte t_srv2;				//tempo do pulso para servo 2
ubyte t_srv3;				//tempo do pulso para servo 3
ubyte t_srv4;				//tempo do pulso para servo 4
ubyte t_srv5;				//tempo do pulso para servo 5
ubyte t_srv6;				//tempo do pulso para servo 6
ubyte t_srv7;				//tempo do pulso para servo 7
ubyte t_srv8;				//tempo do pulso para servo 8

ubyte nr_servo_atual;	//servo atual
ubyte pos_servo_atual;	//posio para servo atual
ubyte maq_est;				//maquina de estado para recepo protocolo
